import hashlib
import json
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend

class TweakedDB:
    def __init__(self, filename, key):
        self.filename = filename
        self.key = self.derive_key(key)
        self.data = {}
        self.load_from_file()

    def derive_key(self, key):
        # Derive a 64-bit key using PBKDF2HMAC
        kdf = PBKDF2HMAC(
            algorithm=algorithms.SHA256(),
            iterations=100000,
            length=8,
            salt=b'salt_for_key_derivation',
            backend=default_backend()
        )
        return kdf.derive(key.encode())

    def hash_key(self, key):
        return hashlib.sha256(key.encode()).hexdigest()

    def encrypt_data(self, data):
        cipher = Cipher(algorithms.AES(self.key), modes.CFB(b'\0' * 16), backend=default_backend())
        encryptor = cipher.encryptor()
        return encryptor.update(data.encode()) + encryptor.finalize()

    def decrypt_data(self, encrypted_data):
        cipher = Cipher(algorithms.AES(self.key), modes.CFB(b'\0' * 16), backend=default_backend())
        decryptor = cipher.decryptor()
        decrypted_data = decryptor.update(encrypted_data) + decryptor.finalize()
        return decrypted_data.decode()

    def set(self, key, value):
        hashed_key = self.hash_key(key)
        encrypted_value = self.encrypt_data(value)
        self.data[hashed_key] = encrypted_value
        self.save_to_file()

    def get(self, key):
        hashed_key = self.hash_key(key)
        encrypted_value = self.data.get(hashed_key, None)
        if encrypted_value:
            return self.decrypt_data(encrypted_value)
        return None

    def delete(self, key):
        hashed_key = self.hash_key(key)
        if hashed_key in self.data:
            del self.data[hashed_key]
            self.save_to_file()

    def get_all_keys(self):
        return [self.decrypt_data(key) for key in self.data.keys()]

    def save_to_file(self):
        with open(self.filename, 'w') as file:
            for key, value in self.data.items():
                file.write(f"{key}:{value}\n")

    def load_from_file(self):
        try:
            with open(self.filename, 'r') as file:
                lines = file.readlines()
                for line in lines:
                    parts = line.strip().split(':')
                    if len(parts) == 2:
                        key, value = parts
                        self.data[key] = value
        except FileNotFoundError:
            pass
